﻿using MudBlazor.Services;
using Netina.Domain.Entities.Products;

namespace Netina.AdminPanel.PWA.Pages;

public class ProductsPageViewModel(
    NavigationManager navigationManager,
    ISnackbar snackbar,
    IUserUtility userUtility,
    IRestWrapper restWrapper,
    IDialogService dialogService,
    IBrowserViewportService browserViewportService)
    : BaseViewModel<ObservableCollection<ProductSDto>>(userUtility)
{
    private readonly NavigationManager _navigationManager = navigationManager;
    private readonly IUserUtility _userUtility = userUtility;

    public string? Search = string.Empty;
    public int CurrentPage = 0;
    public int PageCount = 1;
    public int TotalItems = 0;
    public bool IsXs = false;
    public bool IsEnable { get; set; } = true;

    public override async Task InitializeAsync()
    {
        await GetEntitiesAsync();
        await base.InitializeAsync();
    }

    public async Task GetEntitiesAsync()
    {
        try
        {
            var token = await _userUtility.GetBearerTokenAsync();
            if (token == null)
                throw new Exception("Token is null");
            IsProcessing = true;
            PageDto.Clear();
            var search = Search.IsNullOrEmpty() ? null : Search;
            var dto = await restWrapper.ProductRestApi.ReadAll(CurrentPage, search, SelectedCategory?.Id, IsEnable,token);
            dto.Products.ForEach(d => PageDto.Add(d));
            if (PageDto.Count % 20 == 0)
                PageCount = CurrentPage + 2;
            TotalItems = dto.Pager.TotalItems;
            IsXs = (await browserViewportService.GetCurrentBreakpointAsync()) == Breakpoint.Xs;
        }
        catch (ApiException ex)
        {
            var exe = await ex.GetContentAsAsync<ApiResult>();
            snackbar.Add(exe != null ? exe.Message : ex.Content, Severity.Error);
        }
        catch (Exception e)
        {
            snackbar.Add(e.Message, Severity.Error);
        }
        finally
        {

            IsProcessing = false;
        }
    }

    public async Task ChangePageAsync(int page)
    {
        CurrentPage = page - 1;
        await GetEntitiesAsync();
    }

    public async Task AddProductClicked()
    {
        DialogOptions maxWidth = new DialogOptions() { MaxWidth = MaxWidth.Large, FullWidth = true, DisableBackdropClick = true };
        var breakPoint = await browserViewportService.GetCurrentBreakpointAsync();
        if (breakPoint == Breakpoint.Xs)
            maxWidth = new DialogOptions { FullScreen = true, CloseButton = true };
        var dialogResult = await dialogService.ShowAsync<ProductActionDialogBox>("افزودن محصول جدید", maxWidth);
        var result = await dialogResult.Result;
        if (!result.Canceled && result.Data is bool and true)
        {
            await InitializeAsync();
        }
    }


    public async Task AddFastProductClicked()
    {
        DialogOptions maxWidth = new DialogOptions() { MaxWidth = MaxWidth.Large, FullWidth = true, NoHeader = true, DisableBackdropClick = true, CloseButton = true };
        var breakPoint = await browserViewportService.GetCurrentBreakpointAsync();
        if (breakPoint == Breakpoint.Xs)
            maxWidth = new DialogOptions { FullScreen = true, NoHeader = true, CloseButton = true };
        var dialogResult = await dialogService.ShowAsync<FastProductCreateDialogBox>("افزودن محصول جدید", maxWidth);
        var result = await dialogResult.Result;
        if (!result.Canceled && result.Data is bool and true)
        {
            await InitializeAsync();
        }
    }
    public async Task AddDigikalaProductClicked()
    {

        DialogOptions maxWidth = new DialogOptions() { MaxWidth = MaxWidth.Medium, FullWidth = true, DisableBackdropClick = true };
        var dialogResult = await dialogService.ShowAsync<DigikalaProductActionDialogBox>("افزودن محصول جدید", maxWidth);
        var result = await dialogResult.Result;
        if (!result.Canceled && result.Data is bool and true)
        {
            await InitializeAsync();
        }
    }

    public async Task EditProductClicked(ProductSDto product)
    {
        DialogOptions maxWidth = new DialogOptions() { MaxWidth = MaxWidth.Large, FullWidth = true, DisableBackdropClick = true };
        var breakPoint = await browserViewportService.GetCurrentBreakpointAsync();
        if (breakPoint == Breakpoint.Xs)
            maxWidth = new DialogOptions { FullScreen = true, CloseButton = true };
        var parameters = new DialogParameters<ProductActionDialogBox>();
        parameters.Add(x => x.Product, product);
        var dialogResult = await dialogService.ShowAsync<ProductActionDialogBox>($"ویرایش محصول {product.PersianName}", parameters, maxWidth);
        var result = await dialogResult.Result;
        if (!result.Canceled && result.Data is bool and true)
        {
            await InitializeAsync();
        }
    }

    public async Task DeleteProductAsync(Guid selectedCategoryId)
    {
        var reference = await dialogService.ShowQuestionDialog($"آیا از حذف محصول اطمینان دارید ?");
        var result = await reference.Result;
        if (!result.Canceled)
        {

            try
            {

                IsProcessing = true;
                var token = await _userUtility.GetBearerTokenAsync();
                await restWrapper.CrudDtoApiRest<Product, ProductSDto, Guid>(Address.ProductController)
                    .Delete(selectedCategoryId, token);
                snackbar.Add("حذف محصول با موفقیت انجام شد", Severity.Success);
                await InitializeAsync();

            }
            catch (ApiException ex)
            {
                var exe = await ex.GetContentAsAsync<ApiResult>();
                snackbar.Add(exe != null ? exe.Message : ex.Content, Severity.Error);
            }
            catch (Exception e)
            {
                snackbar.Add(e.Message, Severity.Error);
            }
            finally
            {

                IsProcessing = false;
            }
        }
    }

    public async Task SearchChanged()
    {
        if (Search!=null && !Search.IsNullOrEmpty())
            await InitializeAsync();
    }


    private List<ProductCategorySDto> _productCategories = new List<ProductCategorySDto>();
    public ProductCategorySDto? SelectedCategory;
    public async Task<IEnumerable<ProductCategorySDto>> SearchProductCategory(string category)
    {
        try
        {
            if (category.IsNullOrEmpty())
                _productCategories = await restWrapper.ProductCategoryRestApi.ReadAll(0);
            else
                _productCategories = await restWrapper.ProductCategoryRestApi.ReadAll(category);
            return _productCategories;
        }
        catch (ApiException ex)
        {
            var exe = await ex.GetContentAsAsync<ApiResult>();
            if (exe != null)
                snackbar.Add(exe.Message, Severity.Error);
            snackbar.Add(ex.Content, Severity.Error);
            return _productCategories;
        }
        catch (Exception e)
        {
            snackbar.Add(e.Message, Severity.Error);
            return _productCategories;
        }
    }

    public async Task ProductCategorySelected(ProductCategorySDto productCategory)
    {
        SelectedCategory = productCategory;
        await GetEntitiesAsync();
    }

    public async Task ClearProductCategorySearch()
    {
        SelectedCategory = null;
        await GetEntitiesAsync();
    }

    public async Task DisplayedChanged(ProductSDto product)
    {
        try
        {
            var token = await _userUtility.GetBearerTokenAsync();
            if (token == null)
                throw new Exception("Token is null");
            await restWrapper.ProductRestApi.ChangeDisplayedAsync(product.Id, product.BeDisplayed, token);
        }
        catch (ApiException ex)
        {
            var exe = await ex.GetContentAsAsync<ApiResult>();
            snackbar.Add(exe != null ? exe.Message : ex.Content, Severity.Error);
        }
        catch (Exception e)
        {
            snackbar.Add(e.Message, Severity.Error);
        }
    }

    public async Task ProductEnableChanged(bool? arg)
    {
        if (arg == null)
            return;
        IsEnable = arg.Value;
        CurrentPage = 0;
        await GetEntitiesAsync();
    }

    public async Task ChangeProductCostAsync(ProductSDto product,double cost)
    {

        try
        {
            var token = await _userUtility.GetBearerTokenAsync();
            if (token == null)
                throw new Exception("Token is null");
            await restWrapper.ProductRestApi.ChangeCostAsync(product.Id, cost, token);
            product.Cost = cost;


        }
        catch (ApiException ex)
        {
            var exe = await ex.GetContentAsAsync<ApiResult>();
            snackbar.Add(exe != null ? exe.Message : ex.Content, Severity.Error);
        }
        catch (Exception e)
        {
            snackbar.Add(e.Message, Severity.Error);
        }
    }
}